@@ -10,7 +10,7 @@ use crate::CleanReporter;
 /// Remove a file or directory and all its contents, returning a [`Removal`] with
 /// the number of files and directories removed, along with a total byte count.
 pub fn rm_rf(path: impl AsRef<Path>) -> io::Result<Removal> {
-    Remover::default().rm_rf(path)
+    Remover::default().rm_rf(path, false)
 }
 
 /// A builder for a [`Remover`] that can remove files and directories.
@@ -29,9 +29,13 @@ impl Remover {
 
     /// Remove a file or directory and all its contents, returning a [`Removal`] with
     /// the number of files and directories removed, along with a total byte count.
-    pub(crate) fn rm_rf(&self, path: impl AsRef<Path>) -> io::Result<Removal> {
+    pub(crate) fn rm_rf(
+        &self,
+        path: impl AsRef<Path>,
+        skip_locked_file: bool,
+    ) -> io::Result<Removal> {
         let mut removal = Removal::default();
-        removal.rm_rf(path.as_ref(), self.reporter.as_deref())?;
+        removal.rm_rf(path.as_ref(), self.reporter.as_deref(), skip_locked_file)?;
         Ok(removal)
     }
 }
@@ -52,7 +56,12 @@ pub struct Removal {
 
 impl Removal {
     /// Recursively remove a file or directory and all its contents.
-    fn rm_rf(&mut self, path: &Path, reporter: Option<&dyn CleanReporter>) -> io::Result<()> {
+    fn rm_rf(
+        &mut self,
+        path: &Path,
+        reporter: Option<&dyn CleanReporter>,
+        skip_locked_file: bool,
+    ) -> io::Result<()> {
         let metadata = match fs_err::symlink_metadata(path) {
             Ok(metadata) => metadata,
             Err(err) if err.kind() == io::ErrorKind::NotFound => return Ok(()),
@@ -100,13 +109,25 @@ impl Removal {
                         if set_readable(dir).unwrap_or(false) {
                             // Retry the operation; if we _just_ `self.rm_rf(dir)` and continue,
                             // `walkdir` may give us duplicate entries for the directory.
-                            return self.rm_rf(path, reporter);
+                            return self.rm_rf(path, reporter, skip_locked_file);
                         }
                     }
                 }
             }
 
             let entry = entry?;
+
+            // Remove the exclusive lock last.
+            if skip_locked_file
+                && entry.file_name() == ".lock"
+                && entry
+                    .path()
+                    .strip_prefix(path)
+                    .is_ok_and(|suffix| suffix == Path::new(".lock"))
+            {
+                continue;
+            }
+
             if entry.file_type().is_symlink() && {
                 #[cfg(windows)]
                 {
@@ -121,6 +142,11 @@ impl Removal {
                 self.num_files += 1;
                 remove_dir(entry.path())?;
             } else if entry.file_type().is_dir() {
+                // Remove the directory with the exclusive lock last.
+                if skip_locked_file && entry.path() == path {
+                    continue;
+                }
+
                 self.num_dirs += 1;
 
                 // The contents should have been removed by now, but sometimes a race condition is
