@@ -5,6 +5,7 @@ pub use constraints::*;
 pub use dependency_groups::*;
 pub use dry_run::*;
 pub use editable::*;
+pub use env_file::*;
 pub use export_format::*;
 pub use extras::*;
 pub use hash::*;
@@ -28,6 +29,7 @@ mod constraints;
 mod dependency_groups;
 mod dry_run;
 mod editable;
+mod env_file;
 mod export_format;
 mod extras;
 mod hash;
