@@ -1,19 +1,80 @@
-use crate::Error;
+use std::ffi::{OsStr, OsString};
+use std::path::{Path, PathBuf};
+use std::sync::{Arc, Mutex};
+use std::time::SystemTime;
+
 use fs_err as fs;
 use fs_err::DirEntry;
 use reflink_copy as reflink;
 use rustc_hash::FxHashMap;
 use serde::{Deserialize, Serialize};
-use std::path::{Path, PathBuf};
-use std::sync::{Arc, Mutex};
-use std::time::SystemTime;
 use tempfile::tempdir_in;
 use tracing::{debug, instrument, trace};
-use uv_warnings::warn_user_once;
 use walkdir::WalkDir;
 
+use uv_distribution_filename::WheelFilename;
+use uv_fs::Simplified;
+use uv_preview::{Preview, PreviewFeatures};
+use uv_warnings::{warn_user, warn_user_once};
+
+use crate::Error;
+
+#[allow(clippy::struct_field_names)]
 #[derive(Debug, Default)]
-pub struct Locks(Mutex<FxHashMap<PathBuf, Arc<Mutex<()>>>>);
+pub struct Locks {
+    /// The parent directory of a file in a synchronized copy
+    copy_dir_locks: Mutex<FxHashMap<PathBuf, Arc<Mutex<()>>>>,
+    /// Top level modules (excluding namespaces) we write to.
+    modules: Mutex<FxHashMap<OsString, WheelFilename>>,
+    /// Preview settings for feature flags.
+    preview: Preview,
+}
+
+impl Locks {
+    /// Create a new Locks instance with the given preview settings.
+    pub fn new(preview: Preview) -> Self {
+        Self {
+            copy_dir_locks: Mutex::new(FxHashMap::default()),
+            modules: Mutex::new(FxHashMap::default()),
+            preview,
+        }
+    }
+
+    /// Warn when a module exists in multiple packages.
+    fn warn_module_conflict(&self, module: &OsStr, wheel_a: &WheelFilename) {
+        if let Some(wheel_b) = self
+            .modules
+            .lock()
+            .unwrap()
+            .insert(module.to_os_string(), wheel_a.clone())
+        {
+            // Only warn if the preview feature is enabled
+            if !self
+                .preview
+                .is_enabled(PreviewFeatures::DETECT_MODULE_CONFLICTS)
+            {
+                return;
+            }
+
+            // Sort for consistent output, at least with two packages
+            let (wheel_a, wheel_b) = if wheel_b.name > wheel_a.name {
+                (&wheel_b, wheel_a)
+            } else {
+                (wheel_a, &wheel_b)
+            };
+            warn_user!(
+                "The module `{}` is provided by more than one package, \
+                which causes an install race condition and can result in a broken module. \
+                Consider removing your dependency on either `{}` ({}) or `{}` ({}).",
+                module.simplified_display().green(),
+                wheel_a.name.cyan(),
+                format!("v{}", wheel_a.version).cyan(),
+                wheel_b.name.cyan(),
+                format!("v{}", wheel_b.version).cyan()
+            );
+        }
+    }
+}
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Serialize, Deserialize)]
 #[serde(deny_unknown_fields, rename_all = "kebab-case")]
@@ -48,12 +109,13 @@ impl LinkMode {
         site_packages: impl AsRef<Path>,
         wheel: impl AsRef<Path>,
         locks: &Locks,
+        filename: &WheelFilename,
     ) -> Result<usize, Error> {
         match self {
-            Self::Clone => clone_wheel_files(site_packages, wheel, locks),
-            Self::Copy => copy_wheel_files(site_packages, wheel, locks),
-            Self::Hardlink => hardlink_wheel_files(site_packages, wheel, locks),
-            Self::Symlink => symlink_wheel_files(site_packages, wheel, locks),
+            Self::Clone => clone_wheel_files(site_packages, wheel, locks, filename),
+            Self::Copy => copy_wheel_files(site_packages, wheel, locks, filename),
+            Self::Hardlink => hardlink_wheel_files(site_packages, wheel, locks, filename),
+            Self::Symlink => symlink_wheel_files(site_packages, wheel, locks, filename),
         }
     }
 
@@ -73,18 +135,25 @@ fn clone_wheel_files(
     site_packages: impl AsRef<Path>,
     wheel: impl AsRef<Path>,
     locks: &Locks,
+    filename: &WheelFilename,
 ) -> Result<usize, Error> {
+    let wheel = wheel.as_ref();
     let mut count = 0usize;
     let mut attempt = Attempt::default();
 
-    for entry in fs::read_dir(wheel.as_ref())? {
-        clone_recursive(
-            site_packages.as_ref(),
-            wheel.as_ref(),
-            locks,
-            &entry?,
-            &mut attempt,
-        )?;
+    for entry in fs::read_dir(wheel)? {
+        let entry = entry?;
+        if entry.path().join("__init__.py").is_file() {
+            locks.warn_module_conflict(
+                entry
+                    .path()
+                    .strip_prefix(wheel)
+                    .expect("wheel path starts with wheel root")
+                    .as_os_str(),
+                filename,
+            );
+        }
+        clone_recursive(site_packages.as_ref(), wheel, locks, &entry, &mut attempt)?;
         count += 1;
     }
 
@@ -153,7 +222,10 @@ fn clone_recursive(
 ) -> Result<(), Error> {
     // Determine the existing and destination paths.
     let from = entry.path();
-    let to = site_packages.join(from.strip_prefix(wheel).unwrap());
+    let to = site_packages.join(
+        from.strip_prefix(wheel)
+            .expect("wheel path starts with wheel root"),
+    );
 
     trace!("Cloning {} to {}", from.display(), to.display());
 
@@ -249,6 +321,7 @@ fn copy_wheel_files(
     site_packages: impl AsRef<Path>,
     wheel: impl AsRef<Path>,
     locks: &Locks,
+    filename: &WheelFilename,
 ) -> Result<usize, Error> {
     let mut count = 0usize;
 
@@ -256,10 +329,11 @@ fn copy_wheel_files(
     for entry in WalkDir::new(&wheel) {
         let entry = entry?;
         let path = entry.path();
-
         let relative = path.strip_prefix(&wheel).expect("walkdir starts with root");
         let out_path = site_packages.as_ref().join(relative);
 
+        warn_module_conflict(locks, filename, relative);
+
         if entry.file_type().is_dir() {
             fs::create_dir_all(&out_path)?;
             continue;
@@ -278,6 +352,7 @@ fn hardlink_wheel_files(
     site_packages: impl AsRef<Path>,
     wheel: impl AsRef<Path>,
     locks: &Locks,
+    filename: &WheelFilename,
 ) -> Result<usize, Error> {
     let mut attempt = Attempt::default();
     let mut count = 0usize;
@@ -286,10 +361,11 @@ fn hardlink_wheel_files(
     for entry in WalkDir::new(&wheel) {
         let entry = entry?;
         let path = entry.path();
-
         let relative = path.strip_prefix(&wheel).expect("walkdir starts with root");
         let out_path = site_packages.as_ref().join(relative);
 
+        warn_module_conflict(locks, filename, relative);
+
         if entry.file_type().is_dir() {
             fs::create_dir_all(&out_path)?;
             continue;
@@ -376,6 +452,7 @@ fn symlink_wheel_files(
     site_packages: impl AsRef<Path>,
     wheel: impl AsRef<Path>,
     locks: &Locks,
+    filename: &WheelFilename,
 ) -> Result<usize, Error> {
     let mut attempt = Attempt::default();
     let mut count = 0usize;
@@ -384,10 +461,11 @@ fn symlink_wheel_files(
     for entry in WalkDir::new(&wheel) {
         let entry = entry?;
         let path = entry.path();
-
         let relative = path.strip_prefix(&wheel).unwrap();
         let out_path = site_packages.as_ref().join(relative);
 
+        warn_module_conflict(locks, filename, relative);
+
         if entry.file_type().is_dir() {
             fs::create_dir_all(&out_path)?;
             continue;
@@ -476,7 +554,7 @@ fn symlink_wheel_files(
 fn synchronized_copy(from: &Path, to: &Path, locks: &Locks) -> std::io::Result<()> {
     // Ensure we have a lock for the directory.
     let dir_lock = {
-        let mut locks_guard = locks.0.lock().unwrap();
+        let mut locks_guard = locks.copy_dir_locks.lock().unwrap();
         locks_guard
             .entry(to.parent().unwrap().to_path_buf())
             .or_insert_with(|| Arc::new(Mutex::new(())))
@@ -492,6 +570,18 @@ fn synchronized_copy(from: &Path, to: &Path, locks: &Locks) -> std::io::Result<(
     Ok(())
 }
 
+/// Warn when a module exists in multiple packages.
+fn warn_module_conflict(locks: &Locks, filename: &WheelFilename, relative: &Path) {
+    // Check for `__init__.py` to account for namespace packages.
+    // TODO(konsti): We need to warn for overlapping namespace packages, too.
+    if relative.components().count() == 2
+        && relative.components().next_back().unwrap().as_os_str() == "__init__.py"
+    {
+        // Modules must be UTF-8, but we can skip the conversion using OsStr.
+        locks.warn_module_conflict(relative.components().next().unwrap().as_os_str(), filename);
+    }
+}
+
 #[cfg(unix)]
 fn create_symlink<P: AsRef<Path>, Q: AsRef<Path>>(original: P, link: Q) -> std::io::Result<()> {
     fs_err::os::unix::fs::symlink(original, link)
